/***************************************************************************
 *   Copyright (C) 2008 by Максим Иванов   *
 *   ivanov@cg.ru   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <QString>
#include <QTime>

#include <libxml/uri.h>

#include <stdio.h>
#include <error.h>
#include <errno.h>

const int numValidCharList = 85;
const QString dummyString = "{{{{";


QString encodePassword(const QString& p)
{
  QString sPass = ":";


  if (p.isEmpty())
  {
    return "";
  }


  for (int i = 0; i < p.length(); i++)
  {
    char c = p.at(i).toLatin1();

    const QString sTmp = QString::asprintf("%d:", (c + i + 1));
    sPass += sTmp;
  }


  return sPass;
}


static char validCharList[numValidCharList] =
{
  '!',  '#',  '$',  '%',  '&',  '(', ')',  '*',  '+',  '-',
  '.',  '0',   '1',  '2',   '3',  '4',  '5',  '6', '7', '8',
  '9', ':',  ';',  '<',  '>',  '?',  '@',  'A',  'B', 'C',
  'D',  'E',  'F',  'G',  'H',  'I',  'J',  'K',  'L', 'M',
  'N', 'O',  'P',  'Q',  'R',  'S',  'T', 'U', 'V', 'W',
  'X',  'Y',  'Z',  '[', ']',  '_',  'a',  'b',  'c',  'd',
  'e',  'f',  'g',  'h',  'i',  'j',  'k',  'l',  'm',  'n',
  'o',  'p',  'q',  'r',  's',  't',  'u',  'v',  'w',  'x',
  'y',  'z',  '{',  '|',  '}'
};


static int findCharInList(char c)
{
  int i = -1;


  for (int j = 0; j < numValidCharList; j++)
  {
    if (validCharList[j] == c)
    {
      i = j;


      return i;
    }
  }


  return i;
}


static char getRandomValidCharFromList()
{
  int k = QTime::currentTime().second();


  return validCharList[k];
}


const char* scrambleString(const QString& s)
{
  QString sRet = "";

  if (s.isNull() || s.isEmpty())
  {
    return s.toLatin1().data();
  }

  QString str = encodePassword(s);

  if (str.length() < 32)
  {
    str += dummyString;
   }

  for (int iR = (str.length() - 1); iR >= 0; iR--)
  {
    //
    // Reverse string
    //

    sRet += str.at(iR);
  }

  if (sRet.length() < 32)
  {
    sRet += dummyString;
   }

  int k = getRandomValidCharFromList();
  int l = k + sRet.length() - 2;

  sRet.insert(0, k);

  for (int i1 = 1; i1 < (int)sRet.length(); i1++)
  {
    int j = findCharInList(sRet.at(i1).toLatin1());

    if (j == -1)
    {
      return s.toLatin1().data();
    }

    int i = (j + l * (i1 + 1)) % numValidCharList;

    sRet[i1]=validCharList[i];
  }


  char c = getRandomValidCharFromList() + 2;


  sRet.append(c);
  return (const char *) xmlEncodeEntitiesReentrant(NULL,xmlCharStrdup(sRet.toLatin1().data()));
}

int main (int ac, char *av[])
{
	if (2 == ac)
	{
		printf("%s\n",scrambleString(QString(av[1])));
	}
	else
	{
		error(1, errno, "Invalid number of arguments");
	}
	return 0;
}
